@@ -39,7 +39,7 @@ for my $meth ( qw[conf _lib _perl5lib]) {
 
 =head1 NAME
 
-CPANPLUS::Configure
+CPANPLUS::Configure - configuration for CPANPLUS
 
 =head1 SYNOPSIS
 
@@ -70,14 +70,18 @@ This method returns a new object. Normal users will never need to
 invoke the C<new> method, but instead retrieve the desired object via
 a method call on a C<CPANPLUS::Backend> object.
 
+=over 4
+
 =item load_configs
 
-Controls wether or not additional user configurations are to be loaded 
+Controls whether or not additional user configurations are to be loaded 
 or not. Defaults to C<true>.
 
+=back
+
 =cut
 
-### store teh CPANPLUS::Config object in a closure, so we only
+### store the CPANPLUS::Config object in a closure, so we only
 ### initialize it once.. otherwise, on a 2nd ->new, settings
 ### from configs on top of this one will be reset
 {   my $Config;
@@ -337,7 +341,7 @@ sub save {
     }       
     return unless $self->can_save($file);
 
-    ### find only accesors that are not private
+    ### find only accessors that are not private
     my @acc = sort grep { $_ !~ /^_/ } $self->conf->ls_accessors;
 
     ### for dumping the values
