@@ -1,3 +1,5 @@
+warn "Legacy library @{[(caller(0))[6]]} will be removed from the Perl core distribution in the next major release. Please install the separate libperl4-corelibs-perl package. It is being used at @{[(caller)[1]]}, line @{[(caller)[2]]}.\n";
+
 ;# getopts.pl - a better getopt.pl
 #
 # This library is no longer being maintained, and is included for backward
@@ -7,7 +9,7 @@
 # programming techniques.
 #
 # Suggested alternatives: Getopt::Long  or  Getopt::Std
-#
+
 ;# Usage:
 ;#      do Getopts('a:bc');  # -a takes arg. -b & -c not. Sets opt_* as a
 ;#                           #  side effect.
@@ -16,13 +18,12 @@ sub Getopts {
     local($argumentative) = @_;
     local(@args,$_,$first,$rest);
     local($errs) = 0;
-    local($[) = 0;
 
     @args = split( / */, $argumentative );
     while(@ARGV && ($_ = $ARGV[0]) =~ /^-(.)(.*)/) {
 		($first,$rest) = ($1,$2);
 		$pos = index($argumentative,$first);
-		if($pos >= $[) {
+		if($pos >= 0) {
 			if($args[$pos+1] eq ':') {
 				shift(@ARGV);
 				if($rest eq '') {
